#
#   Vispan - Virus and Spam analyser
#	This program analyses the mail log file entries created by the 
#	MailScanner program written by Julian Field. It is available at
#	http://www.mailscanner.info
#
#   Copyright (C) 2002  David While
#
#   $Id: Mailscanner.pm,v 1.33 2004/12/17 11:14:55 dwhile Exp $
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   The author, David While, can be contacted by email at
#      david.while@uce.ac.uk
#   or by paper mail at
#      David While
#      School of Computing
#      University of Central England
#      Birmingham
#      B42 2SU
#      United Kingdom
#

# Virus scanner search string definitions

package Mailscanner;

my %Scanners = (
  sophos => {
    Name => 'Sophos',
    Output => '>>> Virus',
    String => '>>> Virus \'(.*)\'',
    WhereIs => 'sweep',
    VerFlag => '-v'},
  sophosSAVI => {
    Name => 'Sophos SAVI',
    Output => 'INFECTED::',
    String => 'INFECTED:: (.*)::'},
  inoculan => {
    Name => 'Inoculan',
    Output => 'was infected by virus',
    String => 'was infected by virus \[(.*)\]'},
  clamav => {
    Name => 'ClamAV',
    Output => 'FOUND[ ]*$',
    String => ':.* (.*) FOUND',
    WhereIs => 'clamscan',
    VerFlag => '-V 2>&1',
    VerString => 'ClamAV (.*)/'},
  command => {
    Name => 'Command',
    Output => 'Infection:',
    String => 'Infection: (.*)'},
  "f-prot" => {
    Name => 'F-Prot',
    Output => 'found virus',
    String => 'found virus (.*)',
    WhereIs => 'f-prot',
    VerFlag => '-verno',
    VerString => 'Program version: (.*)'},
  mcafee => {
    Name => 'McAfee',
    Output => 'Found the',
    String => 'Found the (.*) virus !!!',
    WhereIs => 'uvscan',
    VerFlag => '--version',
    VerString => 'Scan engine v(.*?)'},
  "f-secure" => {
    Name => 'F-Secure',
    Output => '.*Infected',
    String => ':.* Infected: (.*)'},
  rav => {
    Name => 'Rav',
    Output => '.*Infected: (.*)',
    String => '.*Infected: (.*)'},
  eTrust => {
    Name => 'eTrust',
    Output => 'is infected by virus:',
    String => 'is infected by virus: (.*)'},
  antivir => {
    Name => 'AntiVir',
    Output => 'ALERT: \[',
    String => 'ALERT: \[(.*?) ',
    WhereIs => 'antivir',
    VerFlag => '--version',
    VerString => 'product version:  (.*?) '},
  bitdefender => {
    Name => 'BitDefender',
    Output => '\/.*infected:',
    String => '\/.*infected: (.*)',
    WhereIs => 'bdc',
    VerFlag => '--info',
    VerString => 'Console v(.*?) '},
  clamavmodule => {
    Name => 'ClamAV Module',
    Output => 'INFECTED',
    String => 'INFECTED::(.*)::'},
  mailscanner => {
    Name => 'Mailscanner',
    Output => 'File(?:nam|typ)e Checks: [^A]',
    String => 'File(?:nam|typ)e Checks: (.*) \(', 
    WhereIs => 'MailScanner',
    VerFlag => '-v',
    VerString => 'MailScanner version (.*)'},
  trend => {
    Name => 'Trend',
    Output => 'Trend found (.*) in ',
    String => 'Trend found (.*) in '},

);

##################################################
#                                                #
# Subroutine: ProcessLine                        #
# Parameters: Line to be processed               #
# Returns: None                                  #
# Purpose: Given a MailScanner log entry,        #
#          Take the appropriate action depending #
#          on what the line is.                  #
#                                                #
##################################################
sub ProcessLine {
   my( $Line ) = @_;

   my($ThisScanner, $IP, %SpamTraps, %elements, $trap, $Score);
   my($bit, $Virus, $Spam, $size, $messages);

   SWITCH: {

   # This is a processing line - extract the number of messages and size
      ($Line =~ /New Batch/i) && do {
         ($messages,$size) = ($Line =~/Scanning (.*?) messages, (.*?) bytes/);
         Stats::AddMessage($messages, $size);
         last SWITCH;
         };

   # This is a spam log entry - find the sending IP address
      ($Line =~ /: Message (.*) from (.*) is [^not]/) && do {
         ($IP) = ($Line =~/from ([0-9\.]*)/);
         $Line =~ s/^(.*)is spam.//;
         @elements = split(/\,/,$Line);
         $Spam = 1;

         # Now split the SpamAssassin report up to get the individual
         # SpamAssassin traps.
         ELEMENT: foreach $bit (@elements) {
            if ( $bit =~ /score/ ) {
               Stats::AddSpamTrap("SpamAssassin");
               ($SpamScore) = ($Line =~ /score=(.*?),/);
               $Line =~ s/^(.*)required (.*?), //;
               @SpamTraps = split(/\,/,$Line);
               TRAP: foreach $trap (@SpamTraps) {
                  $trap =~ s/^ //;
                  # Ignore the autolearn trap
                  if ( $trap =~ /^autolearn/ ) {
                     next;
                     }
                  ($Type,$Score) = split(/ /,$trap);
                  Stats::AddSpamAssassinTrap($Type);
                  }
               last ELEMENT;
               }
            # This is a blacklisted entry
            elsif ( $bit =~ /blacklisted/ ) {
               Stats::AddSpamTrap("Blacklisted");
               last ELEMENT;
               }
               # This is a whitelisted entry
               elsif ( $bit =~ /whitelisted/ ) {
                 Stats::AddSpamTrap("Whitelisted");
                 $Spam = 0;
                 last ELEMENT;
                 } 
            else {
               $bit =~ s/^(\s+)//;
               $bit =~ s/(\s+)$//;
               Stats::AddSpamTrap($bit);
               }
            }
      # Only report it if MailScanner reported it as spam and it was not
      # whitelisted.
      if( $Spam == 1 ) {
         Stats::FoundDomain($IP, $SpamScore, 'S');
         }
      last SWITCH;
      };

      # This is a virus entry - find the virus details from the entry.
      @Vengines = split(' ',Config::Value(Scanner));
      foreach $Scanner (@Vengines) {
         $ThisScanner = $Scanners{$Scanner};
         ($Line =~ /$ThisScanner->{Output}/)   && do {
            ($Virus) = ($Line =~ /$ThisScanner->{String}/);
            # Add one to the virus score.
            if( $Virus ne "" ) {
               Stats::AddVirus($Virus, $ThisScanner->{Name});
	       last SWITCH;
               }
            }
         }

      # Virus infected get the IP address
      ($Line =~ /Infected message/i) && do {
         ($IP) = ($Line =~ /came from (.*)/);
         $IP =~ s/\s//g; #remove whitespace 
         Stats::FoundDomain($IP, 0, 'V');
         last SWITCH;
         }
      }
   }


##################################################
#                                                #
# Subroutine: PrintScannerNames                  #
# Parameters: None                               #
# Returns: None                                  #
# Purpose: Used in debug mode to print the full  #
#          names of the scanners                 #
#                                                #
##################################################
sub printScannerNames {
  my($ThisScanner);

  @Vengines = split(' ',Config::Value(Scanner));
  foreach $Scanner (@Vengines) {
    $ThisScanner = $Scanners{$Scanner};
    print "Scanner Name of '" . $Scanner . "' = '" . $ThisScanner->{Name} . "'\n";
    }
  }

##################################################
#                                                #
# Subroutine: GetLocalVerNums                    #
# Parameters: None                               #
# Returns: None                                  #
# Purpose: Get the version numbers of the        #
#          scanners                              #
#                                                #
##################################################
sub GetLocalVerNums {
   my($ThisScanner, $VerTest, $VerString, $TestProg, $res, $program, $ver);
   my $Debug = Config::Value(Debug) || 0;
   my $Whereis = Config::Value(WhereIs);
   my $WhereisPath = Config::Value(WhereIsPath);

   # First get the version of spamassassin
   $TestProg = 'spamassassin';
   $VerTest = '-V | sed -e "s/.*version \(.*\)/\1/"';

   $res = `$Whereis -B $WhereisPath -f $TestProg`;
   print "whereis=$res" if($Debug);
   ($result) = ($res =~ /: ([\w|\/|\-]*)/);
   print "Prog=$result\n" if($Debug);
   $res = `$result $VerTest`;
   print "ver=$res" if($Debug);
   ($res) = ($res =~ /([\w\.\-]*)/);
   print "$TestProg=$res\n" if($Debug);
   Stats::AddVirVersion($TestProg, $res) if( $res ne "" );

   # Now step through each scanner specified in the config file.
   @Vengines = split(' ',Config::Value(Scanner));
   foreach $Scanner (@Vengines) {
      $ThisScanner = $Scanners{$Scanner};
      $TestProg = $ThisScanner->{WhereIs};
      $VerFlag = $ThisScanner->{VerFlag};
      $VerString = $ThisScanner->{VerString};
      $res = ""; $ver="";
      if( $TestProg ne "") {
         $res = `$Whereis -B $WhereisPath -f $TestProg`;
         print "whereis=$res" if($Debug);
         ($program) = ($res =~ /: ([\w|\/|\-]*)/);
         print "Prog=$program\n" if($Debug);

         if( $VerFlag ne "" ) {
            $res = `$program $VerFlag`;
            ($ver) = ($res =~ /$VerString/);
            }
         }

      print "ver=$ver\n" if($Debug);
      print "$Scanner=$ver\n" if($Debug);
      Stats::AddVirVersion($Scanner, $ver) if( $ver ne "" );
      }
   }

1;
